Imports System.IO
Imports System.Net.Sockets
Imports System.Net

    '---------------------------------------------------------------------------------------
    'Instructions for use:
    '1. Create an event and a trigger using HS3. 
    '2. Click the "Advanced Options" icon. 
    '3. In the Sub or Function field, enter: "runCycle" or "runNamedCycle" (no quotes, exact case)
    '4. If using runCycle, in the parameters field enter:  ControllerName; password; 2;2;0;0;0;0;0;0
    '5. If using runNamedCycle, in parameters field enter:  ControllerName; cyclename
    '6. Make sure to match the case when entering text for controller name and password.
    '7. Make sure to use the semi-colon characters where shown in step 4 or 5.
    '8. Don't modify this script.  See EtherRain Interface FunctionReference (pdf) for more info
    '----------------------------------------------------------------------------------------
    Public Structure deviceStruct
        Public devname As String
        Public pw As String
        Public zonename() As String
        Public location As String
        Public status As Boolean
        Public del As Boolean   'true if del
        Public ipPort As String
    End Structure
    Public Const discocmd As String = "eviro_id_request1"
    Public Const productkey As String = "eviro t="
    'default port set to different port than search port, allows operation with adminprogram
    Public Const defRport As Integer = 8089  'discovery broadcast port
    Public Const configPage As String = "ergetcfg.cgi"
    Public Const resultPage As String = "result.cgi"
    Public bcport As Long  'discovery broadcast port
    Public myHostName As String
    Public myIPaddress As String
    Public sharedmsg As String
    Public webstr As String
    Public myurl As String
    Public mycycleURL As String
    Public myquerystring As String
    Public deviceTot As Integer
    Public deviceNdx As Integer
    Public deviceSelected As String
    Public erdevices() As deviceStruct
    Public controlAry() As String   'used in function readControllers/Main

    Public Sub Main(ByVal Parms As Object)
        '
        '   Version Information:  EtherRain.vb Revision 1.4 January 2 2014
        '
        '
        ' --------------------------------------------------------------------------------
        '
        '                          IMPORTANT USAGE INFORMATION:
        '  THIS IS FOR HOMESEER 3 ONLY -  not for use with HomeSeer2
        '
        ' This script, EtherRain.vb is provided for use by HomeSeer HomeSeer 3 Server owners.
        ' HomeSeer software developers may also use this script, in total or in part, for 
        ' purposes of extending functions and capabilities on the HomeSeer 3 server 
        ' platform, subject to the following terms:
        ' 1. If this script is modified in any way, it may not be re-distributed
        '    unless it is re-named.
        ' 2. If the script is re-sold, either modified or unmodified, it must be 
        '    renamed and the distributor/re-seller assumes complete responsibility for 
        '    all support.
        ' 3. Only Scripts downloaded directly from QuickSmart Development's web site 
        '    are supported by QuickSmart
        ' 4. If a developer or individual wishes to provide enhanced functionality 
        '    within the original EtherRain.vb script, but does not wish to provide 
        '    support, they may submit their add-on functions and subroutines to QuickSmart,
        '    and, if accepted, QuickSmart will add the functionality to the base script 
        '    and then provide support for those extensions.   
        '    Any functions submitted for inclusion in the base script must not be  
        '    subject to copyright.  Attribution of add-on functionality will be   
        '    given to the author/developer by name, though QuickSmart reserves the right 
        '    to make changes to the function for purposes of script management.
        '
        '  7-23-09 getNamedCycle formating relaxed
        '  7-25-09 UDP socket loop added
        '  8-5-2011 Added functions:  disableSystem; enableSystem; suspendSystem for event calls
        '  6-10-2012 added functions runSeqCycles (runSeqCyclesRet) and getScaledRunTime  - contributed by kanga
        '  6-10-2012 runCycle and runNamedCycle now check status before sending a command; (fixed slight "rain" bug)
        '  11-05-2012 an update was made to runCycleRet - the status check was moved infront of getevirodevices to prevent overwrite of ip address
        '  12-31-2013 tested with HomeSeer 3 - changes noted in the functions
        '  12-31-2013 Sequential cycles has been commented out until code for setting events is developed
        '  07-15-2017 Alert builder fixed for HS3
        '  07-15-2017 Log prefixes shortened to:  EthrRn: {RunSeq/RunCy/RunNaCy/Alerts}
        '  07-21-2017 Sequential cycles Fixed for HS3
        '  07-22-2017 Reset commmand fixed.  (showed up for Rev C controllers only)
        ' --------------------------------------------------------------------------------
        '
        'the Main sub is used to provide Controller Monitoring
        'this sub is called by the EtherRain Alert event which is 
        'configured by the user from the EtherRain asp scripts
        'if called, the er_alertbuilder page would have set up the parameters
        'and er_home would have created the event
        'this function is then executed only by event

        Dim i, j As Integer
        Dim wstr, mess As String
        Dim dcount, dfound As Integer
        Dim smins As Integer
        Dim emailalert, logalert As String
        Dim params() As String
        Dim responded As Boolean
        Dim d As String = Format(Today, "MM-dd-yyy")
        Dim CurrentTime As DateTime
        Dim sCurrentTime As String

        Dim sendemail As Boolean
        Dim emailaddress As String
        Dim emailtitle, emailbody As String
        Dim mCCstr, mBCCstr, attStr As String

        mCCstr = ""
        mBCCstr = ""
        attStr = ""

        CurrentTime = DateTime.Now
        sCurrentTime = CurrentTime.ToString("H:mm")  ' if you want the colon

        'initialize email
        emailtitle = "HomeSeer EtherRain Device Alert"
        emailbody = "Results of EtherRain Contact Event on " & d & " at " & sCurrentTime & vbCrLf & vbCrLf
        emailbody = emailbody & "The following controllers did not respond to a status request: " & vbCrLf

        '1. get the alert configuration ini
        logalert = "off"
        emailalert = "off"
        emailaddress = ""
        sendemail = False
        wstr = readAlertConfig()
        If Len(wstr) > 0 Then
            params = Split(wstr, ";")
            If UBound(params) = 3 Then
                emailaddress = params(0)
                smins = Convert.ToInt32(params(1))
                emailalert = params(2)  'on or off
                logalert = params(3)    'on or off
            Else
                mess = "Error:  Parameter Error in EtherRain Alerts section."
                hs.WriteLog("EthrRn:Alerts", mess)
                Exit Sub
            End If
        Else
            mess = "Error:  No EtherRain Alerts Parameters found."
            hs.WriteLog("EthrRn:Alerts", mess)
            Exit Sub
        End If

        '2. get the controller definination ini - these will be in controlAry()
        dcount = readControllers()  'this reads the hs. controller definitions
        If dcount = 0 Then
            mess = "Error:  No EtherRain Controllers defined in HomeSeer."
            hs.WriteLog("EthrRn:Alerts", mess)
            Exit Sub
        End If
        'we only need the controller name, so ditch the rest of the array
        For i = 0 To dcount - 1
            params = Split(controlAry(i), ";")
            controlAry(i) = params(0)
        Next
        'controlAry(i) now holds only the controller HS defined controller name
        '3. send a discovery to the devices
        'now  get the controller IP address
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            sharedmsg = "Error: " & sharedmsg
            hs.WriteLog("EthrRn:Alerts", sharedmsg)
            Exit Sub
        End If

        'rethink this; if no devices found we need a log entry or email
        'for the devices not found
        'send only one email containing all devices not found
        dfound = erdevices.Length

        For i = 0 To dcount - 1
            responded = False
            For j = 0 To dfound - 1
                If controlAry(i) = erdevices(j).devname Then
                    responded = True
                    Exit For
                End If
            Next
            If responded Then
                If logalert = "on" Then
                    mess = " " & erdevices(j).devname & " is responding OK. (" & erdevices(j).ipPort & ")"
                    hs.WriteLog("EthrRn:Alerts", mess)
                End If
            Else
                If logalert = "on" Then
                    mess = " " & controlAry(i) & " is NOT currently Responding."
                    hs.WriteLog("EthrRn:Alerts", mess)
                End If
                If emailalert = "on" Then
                    sendemail = True
                    emailbody = emailbody & controlAry(i) & "  is NOT currently responding" & vbCrLf
                End If

            End If
        Next
        If sendemail Then
            If Len(emailaddress) = 0 Then
                mess = "Alerts Error: Email address was not found."
                hs.WriteLog("EthrRn:Alerts", mess)
                Exit Sub
            End If
            mess = "Email Alert sent to: " & emailaddress
            hs.WriteLog("EthrRn:Alerts", mess)
            emailbody = emailbody & "Please check these controllers to ensure they have power and a network connection." & vbCrLf
            'hs.SendEmail(emailaddress, emailaddress, emailtitle, emailbody)
            hs.SendEmail(emailaddress, emailaddress, mCCstr, mBCCstr, emailtitle, emailbody, attStr)
        End If



    End Sub
    Public Sub getStatus(ByVal pstr As Object)
        'used only for debugging
        Dim rstr As String
        rstr = getStatusRet(pstr)

    End Sub

    Public Function getStatusRet(ByVal pstr As Object) As String

        Dim i, j, k, dfound As Integer
        Dim devnamestr, devpwstr, mess As String
        Dim responded As Boolean
        Dim responsestr As String
        Dim qstr, wstr As String
        Dim rez As Boolean
        Dim url, myurl As String
        ' Dim pary As String


        'this gets the status of a device 
        ' The parameter string contains 1 parameter
        ' The parameter is:  Controller name
        ' This method will write a status string to the log
        ' This method will return the status
        ' This function was designed to be called using s=hs.RunEx()

        'call getstatus from an asp page titled er_getStatus.asp
        'call er_getStatus.asp from the ajax control
        'er_getStatus.asp only returns one line of text - the output of this function!
        'the input to this function is the controller name
        'this function does the discovery to check whether the name is found
        'if found, it trys logging in to test the password
        'if the password works, it then returns a device status (rd, wt, bz)
        'returns:
        '  No Response from NAME
        '  Working.  NAME is {Ready/Waiting/Busy}
        '  if working, get status bits:  rain/ short detected

        If Len(pstr) = 0 Then
            mess = "No Name Provided"
            getStatusRet = mess
            Exit Function
        End If

        Dim pary() = pstr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length
        If UBound(pary) = 1 Then
            devnamestr = Trim(pary(0))
            devpwstr = Trim(pary(1))
        Else
            devnamestr = Trim(pary(0))
        End If
        'I think only the devname is required to get status
        'now  get the controller IP address
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            'sharedmsg = "Error: " & sharedmsg
            getStatusRet = sharedmsg
            Exit Function
        End If

        dfound = erdevices.Length
        responded = False
        For j = 0 To dfound - 1
            If Trim(devnamestr) = Trim(erdevices(j).devname) Then
                responded = True
                Exit For
            End If
        Next
        If responded = False Then
            mess = "No response from " & devnamestr
            getStatusRet = mess
            Exit Function
        End If
        'we have a response.

        responsestr = devnamestr & " Working. Status: "
        'now need to get status (ready/busy/waiting/rain/short)
        'setup the url
        myurl = "http://" & erdevices(j).ipPort
        'also, make sure the password is accepted
        qstr = "/" & resultPage & "?" & "xs"
        url = myurl & qstr
        rez = getPage(url)
        If Not rez Then
            getStatusRet = responsestr & "Unknown"
            Exit Function
        End If
        k = InStr(webstr, "os:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr = "BZ" Then
            getStatusRet = responsestr & "Running"
            Exit Function
        End If
        If wstr = "WT" Then
            getStatusRet = responsestr & "Waiting"
            Exit Function
        End If
        If wstr = "RD" Then
            responsestr = responsestr & "Ready"

        End If
        'but now check  short
        k = InStr(webstr, "rz:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr = "SH" Then
            getStatusRet = responsestr & "(Short)"
            Exit Function
        End If

        'but now check  rain
        k = InStr(webstr, "rn:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 1)
        End If
        If wstr = "1" Then
            responsestr = responsestr & "(Rain)"
        End If


        getStatusRet = responsestr

    End Function


    Public Sub runSeqCycles(ByVal pstr As Object)
        'homeSeer events can only call subroutines directly
        'while the homeseer RunEx command calls functions
        'this sub is a wrapper for the function
        'the sub is called from a homeseer event action
        'the function is called from the homeseer RunEX function
        Dim i, j, istatus As Integer
        Dim rstr As String
        Dim stVar, mess As String
        Dim section As String
        Dim items() As String
        Dim tparams() As String
        Dim suspenddate As String
        Dim d_today, d_sus As Date

        'disable and suspend have now been integrated into the EtherRain interface so first check system status
        stVar = ""
        section = hs.GetINISection("system status", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "status=")
                If j > 0 Then
                    stVar = Mid(items(i), j + 7)
                    Exit For
                End If
            Next
        End If
        'split the status str into the global vars and construct statusstr
        istatus = 0      'system is enabled
        suspenddate = ""  'no date
        If Len(stVar) > 0 Then
            tparams = Split(stVar, ";")
            If UBound(tparams) = 1 Then
                istatus = CInt(tparams(0))
                suspenddate = tparams(1)
            End If
            If UBound(tparams) = 0 Then
                istatus = CInt(tparams(0))
            End If
        End If

        'If istatus = 0 Then  we are enabled
        If istatus = 1 Then
            'check the date, if today's date is equal or greater, then continue
            'statusstr = "Suspended by User until " & suspenddate
            d_today = Date.Now
            d_sus = CDate(suspenddate)
            If d_today < d_sus Then
                mess = "EtherRain Irrigation is Currently Suspended by User"
                hs.WriteLog("EthrRn:SeqCy", mess)
                Exit Sub
            End If
        ElseIf istatus = 2 Then
            mess = "EtherRain Irrigation is Currently Disabled by User"
            hs.WriteLog("EthrRn:SeqCy", mess)
            Exit Sub

        End If

        'do a quick check to ensure the command is written correctly
        'just make sure that there are an even number of elements in the paramter string
        Dim pary() = pstr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length
        If (i < 2) Then
            mess = "Error: Not enough elements found in parameter string. Check parameter string for proper delimiting."
            hs.WriteLog("EthrRn:SeqCy", mess)
            Exit Sub
        End If
        If (i Mod 2 <> 0) Then
            mess = "Element Mismatch; Please specify a named-cyle for every controller. Check parameter string."
            hs.WriteLog("EthrRn:SeqCy", mess)
            Exit Sub
        End If
        'changed limit to 18 - nine sequential cycles July,2017
        If (i > 18) Then
            mess = "Limit of 18 parameters in list has been exceeded."
            hs.WriteLog("EthrRn:SeqCy", mess)
            Exit Sub
        End If

        rstr = runSeqCyclesRet(pstr)
        hs.WriteLog("EthrRn:SeqCy", rstr)

    End Sub

    Public Function runSeqCyclesRet(ByVal pStr As Object) As String
        ' runSeqCyclesRet immediately runs the first cycle in the list and sets up events for the remaining cycles 
        ' Calculates the cycle run times (including any monthly setbacks) and schedules cycles for sequential running
        ' scheduled cycles self delete after trigger
        ' The parameter string consists of a list of controller names and cycle names 
        ' The controller name and named cycle must be defined and existing in the .ini file
        ' Named cycles are created using the er_cyclebuilder.asp web page
        ' see function guide for more information
        ' max length of 9 sequential cycles ( 18 parameters in the list) - not enforced
        ' June 4, 2012, updated July 2017

        Dim i As Integer
        Dim rstr As String
        Dim mess As String
        'Dim CurrentTime As DateTime
        Dim cycleStartTime As DateTime
        Dim nextRunStartTime As DateTime
        'Dim sNextRunStartTime As String
        'Dim sNextStartDay As String
        Dim controllerAry(), cycleNameAry() As String
        'Dim ScriptString As String
        Dim pary() As String
        Dim scaledRunTime As Integer
        'Dim scaledRunTimeHR As Integer
        'Dim scaledRunTimeMins As Integer
        Dim cycle As Integer
        Dim cyMax As Integer

        Dim eventRef As Integer
        Dim rez As Boolean
        'Dim tValue As TimeSpan
        Dim cycleLength As Integer
        Dim eventName As String
        'Dim startDateTime As Date
        Dim paramStr As String

        scaledRunTime = 0
        'scaledRunTimeHR = 0
        'First of all split input params into an array
        pary = pStr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length
        ' rstr = "Number of controllers in command is: " & Str(pary.Length)
        'runSeqCyclesRet = rstr
        'Exit Function

        If (i < 2) Then
            rstr = "Error: Not enough elements found in parameter string. Check parameter string for proper delimiting."
            runSeqCyclesRet = rstr
            Exit Function
        End If
        If (i Mod 2 <> 0) Then
            rstr = "Error:  Parameter mismatch; Please specify a named-cyle for every controller."
            runSeqCyclesRet = rstr
            Exit Function
        End If
        cyMax = i \ 2   ' given above test, i will always be divisible by 2
        ReDim controllerAry(cyMax - 1)
        ReDim cycleNameAry(cyMax - 1)
        cycle = 0
        For i = 0 To i - 1 Step 2
            controllerAry(cycle) = Trim(pary(i))
            cycleNameAry(cycle) = Trim(pary(i + 1))
            cycle = cycle + 1
        Next

        'the controller names and cycles are now in their array vars   
        For i = 0 To cyMax - 1
            ' Get total run time adjusted for setback from the i'th (0,2,4...)controller to
            ' schedule the next controller start time.
            ' Add 1 minute to cover the 6 second inter-valve delay * 7 = 42 seconds.
            'scaledRunTimeHR = 0
            'scaledRunTimeMins = 0
            scaledRunTime = getScaledRunTime(cycleNameAry(i)) + 1

            'mess = "Run time for " & cycleNameAry(i) & " = " & scaledRunTime
            'hs.WriteLog("EthrRn:SeqCy", mess)

            'If scaledRunTime > 60 Then
            '    scaledRunTimeHR = Convert.ToInt32(scaledRunTime / 60)
            '    scaledRunTimeMins = scaledRunTime - (scaledRunTimeHR * 60)
            'Else
            '    scaledRunTimeHR = 0
            '    scaledRunTimeMins = scaledRunTime
            'End If

            ' now run or schedule the event
            If i = 0 Then
                ' Run the first controller immediately, 
                If (scaledRunTime < 2) Then
                    mess = "Cycle run time is 0 for " & controllerAry(i) & " ; " & cycleNameAry(i) & " - ignoring"
                    hs.WriteLog("EthrRn:SeqCy", mess)
                Else
                    'This is the FIRST CYCLE of the sequence
                    'mess = "First 2 parameters in string = " & Controller & " ; " & Cycle
                    'hs.WriteLog("CHEtherRain:runCHSeqCycle", mess)
                    'Dim tValue As TimeSpan
                    'ScriptString = "(" & """runNamedCycle""" & "," & """" & controllerAry(i) & ";" & cycleNameAry(i) & """" & ")"
                    'mess = "Script string = " & ScriptString

                    cycleStartTime = Date.Now

                    runNamedCycle(controllerAry(i) & ";" & cycleNameAry(i))

                    mess = "Cycle " & Trim(Str(i + 1)) & " for " & controllerAry(i) & " starting: " & cycleStartTime.ToString("H:mm:ss") & "; Runtime: " & scaledRunTime & " min."
                    hs.WriteLog("EthrRn:SeqCy", mess)
                    'tValue = New TimeSpan(0, scaledRunTimeHR, scaledRunTimeMins, 10)  'the 10 is 10 extra seconds

                    nextRunStartTime = cycleStartTime.AddMinutes(scaledRunTime)
                    'add one minute to make up for interzone delays  - the one is added above when the scaled runtime is calced
                    'nextRunStartTime = nextRunStartTime.AddMinutes(1)
                    'sNextRunStartTime = nextRunStartTime.ToString("H:mm")
                    'sNextStartDay = nextRunStartTime.ToString("d")

                End If
            Else
                'set up the temporary event for this cycle; calculate next start time for next scheduled cycle
                If scaledRunTime < 2 Then
                    mess = "Cycle run time is 0 for " & controllerAry(i) & " ; " & cycleNameAry(i) & " - ignoring"
                    hs.WriteLog("EthrRn:SeqCy", mess)
                Else
                    'ALL AFTER THE FIRST CYCLES of the sequence

                    'ScriptString = "EtherRain.vb(" & """runNamedCycle""" & "," & """" & controllerAry(i) & ";" & cycleNameAry(i) & """" & ")"
                    'mess = "Script string = " & ScriptString
                    'hs.WriteLog("EtherRain:runSeqCycles", mess)

                    'following two lines are original event creation lines - automatically deleted event after trigger
                    'hs.NewTimeEvent(eventName, sNextRunStartTime, sNextStartDay, 1, 1, 1, 1, 1, 1, 1, "", 1, ScriptString, "Temporary EtherRain")
                    'hs.EnableEvent(eventName)

                    'here is the documentation for the AddActionRunScript call:
                    'Public Sub AddActionRunScript(ref As Integer, script As String, method As String, parms As String)
                    'https//: forums.homeseer.com/bugzilla/show_bug.cgi?id=1398

                    cycleStartTime = nextRunStartTime
                    cycleLength = scaledRunTime '+ 1  'add 1 minutes to make sure no overlap  - the 1 is added when the scaled time is calced
                    paramStr = controllerAry(i) & ";" & cycleNameAry(i)
                    eventName = "Sequential cycle " & Trim(Str(i + 1)) & " (" & controllerAry(i) & ")"

                    ' create a new event in hs3
                    Try
                        eventRef = hs.NewEventGetRef(eventName, "EtherRain", "Temporary EtherRain")
                        ' set the trigger time of the event 
                        rez = hs.EventSetTimeTrigger(eventRef, cycleStartTime)
                        ' set the event to delete itself after it runs
                        hs.DeleteAfterTrigger_Set(eventRef)
                        ' add a device action to the event
                        hs.AddActionRunScript(eventRef, "EtherRain.vb", "runNamedCycle", paramStr)
                        'Public Sub AddActionRunScript(ref As Integer, script As String, method As String, parms As String)
                        hs.EnableEventByRef(eventRef)
                    Catch ex As Exception
                        hs.WriteLog("EthrRn:seqCy", "Error Creating Event " & eventName)
                        hs.WriteLog("EthrRn:seqCy", ex.Message)
                    End Try

                    mess = "Cycle " & Trim(Str(i + 1)) & " for " & controllerAry(i) & " starting " & cycleStartTime.ToString("H:mm:ss") & "; Runtime: " & cycleLength & " min."
                    hs.WriteLog("EthrRn:SeqCy", mess)



                    'tValue = New TimeSpan(0, scaledRunTimeHR, scaledRunTimeMins, 10)
                    'nextRunStartTime = nextRunStartTime.Add(tValue)
                    nextRunStartTime = cycleStartTime.AddMinutes(cycleLength)
                    'sNextRunStartTime = nextRunStartTime.ToString("H:mm")
                    'sNextStartDay = nextRunStartTime.ToString("d")
                End If

            End If
        Next
        runSeqCyclesRet = Trim(Str(cyMax)) & " irrigation cycles scheduled."

    End Function

    Private Function getScaledRunTime(ByVal cyName As String) As Integer
        Dim i, j As Integer
        Dim mess As String
        Dim cyclestr As String
        Dim testary() As String
        Dim ipercent As Integer
        Dim zonelimit As Integer = 8
        Dim totalscaledMinutes As Integer

        'added 5-10-2012 - Initial codebase provided by "Kanga"
        'this function is only called from runSeqCyclesRet
        'function returns scaled cycle minutes for month if scale/setback is applied to cycle
        'function returns total cycle minutes if setback is not applied to the cycle
        'establish a zonelimit so we don't send time to undefined zones
        'no need to test controller name and pw here, we just want the cycle run times
        'if the Named cycle not found: returns "X-" and an error message
        cyclestr = getNamedCycle(cyName)
        If Len(cyclestr) = 0 Then
            mess = "Error: No cycle information found for " & cyName
            hs.WriteLog("EthrRn:seqCy", mess)
            getScaledRunTime = 0  'this function must return an integer
            Exit Function
        End If
        If Mid(cyclestr, 1, 2) = "X-" Then
            mess = "Error: " & Mid(cyclestr, 3)
            hs.WriteLog("EthrRn:seqCy", mess)
            getScaledRunTime = 0   'this function must return an integer
            Exit Function
        End If
        'ensure the cyclestr is correctly formatted, need to see 8 values
        testary = Split(cyclestr, ":")
        i = UBound(testary)
        If UBound(testary) <> 8 Then
            mess = "Error: " & cyclestr & " cycle is malformed: " & cyclestr
            hs.WriteLog("EthrRn:seqCy", mess)
            getScaledRunTime = 0  'this function must return an integer
            Exit Function
        End If
        'impose the zone limit - zero's all time above the zone limit
        cyclestr = ""
        For i = 0 To 7
            If i < zonelimit Then
                cyclestr = cyclestr & testary(i)
            Else
                cyclestr = cyclestr & "0"
                testary(i) = "0"  'need to zero out this for setback below
            End If
            If i < 7 Then
                cyclestr = cyclestr & ":"
            End If
        Next

        'we can now be sure that cyclestr is properly formed
        'now we need to see if we need to apply the setback, if not, use 100%
        If testary(8) = "SB" Then
            'get the date and the setback
            ipercent = getSetBackMonth() 'gets the setback for the month we are in
        Else
            ipercent = 100
        End If

        totalscaledMinutes = 0
        'multiply the values by the percent to get new values
        For i = 0 To 7
            j = (Convert.ToInt32(testary(i)) * ipercent) \ 100
            If ipercent > 0 And j = 0 And Val(testary(i)) > 0 Then
                j = 1  'this is the minimum if setback is not zero
            End If
            'testary(i) = Trim(Str(j))
            totalscaledMinutes = totalscaledMinutes + j   ' testary(i) mm is an integer
        Next
        getScaledRunTime = totalscaledMinutes
    End Function


    Public Sub runNamedCycle(ByVal pstr As Object)
        'homeSeer events can only call subroutines directly
        'while the homeseer RunEx command calls functions
        'this sub is a wrapper for the function
        'the sub is called from a homeseer event action
        'the function is called from the homeseer RunEX function
        Dim i, j, istatus As Integer
        Dim rstr As String
        Dim stVar, mess As String
        Dim section As String
        Dim items() As String
        Dim tparams() As String
        Dim suspenddate As String
        Dim d_today, d_sus As Date


        'disable and suspend have now been integrated into the EtherRain
        'interface.  before sending the request on, get the system status
        'to make sure it is enabled.
        'if diabled or suspended, send a log entry indicating so.

        stVar = ""
        section = hs.GetINISection("system status", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "status=")
                If j > 0 Then
                    stVar = Mid(items(i), j + 7)
                    Exit For
                End If
            Next
        End If
        'split the status str into the global vars and construct statusstr
        istatus = 0      'system is enabled
        suspenddate = ""  'no date
        If Len(stVar) > 0 Then
            tparams = Split(stVar, ";")
            If UBound(tparams) = 1 Then
                istatus = CInt(tparams(0))
                suspenddate = tparams(1)
            End If
            If UBound(tparams) = 0 Then
                istatus = CInt(tparams(0))
            End If
        End If

        'If istatus = 0 Then  we are enabled
        If istatus = 1 Then
            'check the date, if today's date is equal or greater, then continue
            'statusstr = "Suspended by User until " & suspenddate
            d_today = Date.Now
            d_sus = CDate(suspenddate)
            If d_today < d_sus Then
                mess = "EtherRain Irrigation is Currently Suspended by User"
                hs.WriteLog("EthrRn:RunNaCy", mess)
                Exit Sub
            End If
        ElseIf istatus = 2 Then
            mess = "EtherRain Irrigation is Currently Disabled by User"
            hs.WriteLog("EthrRn:RunNaCy", mess)
            Exit Sub

        End If



        'either we are enabled, or the suspension period has passed so proceed
        rstr = runNamedCycleRet(pstr)


    End Sub


    Public Function runNamedCycleRet(ByVal pStr As Object) As String
        ' runNamedCycleRet sends a command to an EtherRain Valve Controller
        ' The parameter string contains 2 parameters each separated by a semicolon
        ' The parameters are:  Controller name,  and a Named Cycle.
        ' (Controller name;  NamedCycle )
        ' The named cycle must be defined and existing in the .ini file
        ' The Controller must be defined and existing in the .ini file 
        ' The controller password is extracted from the controller section of the .ini file
        ' Named cycles are created using the er_cyclebuilder.asp web page
        ' parse the pStr here
        ' 6-2-2012  first check the rain status;  if rain sensor is on then log message and quit
        ' 11-05-2012  - no bug to fix on this one; the status is checked before the structure is searched

        Dim i, j, dsel As Integer
        Dim ij As Integer
        Dim mess As String
        Dim devStat As String
        Dim devnamestr, devpwstr, icmdstr, cyclestr, result As String
        Dim dcount As Integer
        Dim params() As String
        Dim foundboo As Boolean
        Dim testary() As String
        Dim ipercent As Integer
        Dim zonelimit As Integer = 8



        'cannot use commas as the delimiter because VB parses into multiple params
        'use semi-colons instead
        dsel = 0  'dsel is device selected

        ' Split the string at the comma characters and add each field to a ListBox
        Dim pary() = pStr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length


        If (i <> 2) Then
            mess = "Parameter Mismatch, " & Str(i) & " provided, 2 required."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        'do these meet the requirements for controller name and password?
        'if yes, then get the minutes array and construct the irrigation string
        devnamestr = Trim(pary(0))
        devpwstr = ""
        'get the password here from the saved controllers field
        'get the stored devices here, compare controller name entered 
        'to controller device named stored, if found, get the password
        dcount = readControllers()  'this reads the hs. controller definitions
        If dcount = 0 Then
            mess = "Error:  No EtherRain Controllers defined in HomeSeer."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        'we only need the controller name and password, so ditch the rest of the array
        foundboo = False
        For i = 0 To dcount - 1
            params = Split(controlAry(i), ";")

            If devnamestr = Trim(params(0)) Then
                devpwstr = Trim(params(1))
                devnamestr = Trim(params(0))
                For j = 2 To UBound(params)
                    If Len(Trim(params(j))) = 0 Then
                        zonelimit = j - 2
                        If zonelimit < 0 Then zonelimit = 0
                        Exit For
                    End If
                Next
                foundboo = True
                Exit For
            End If
        Next
        If foundboo = False Then
            mess = "Error:  Provided Controller Name does not match Stored Controller Name."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        If zonelimit = 0 Then
            mess = "Error:  Cycle " & pary(1) & "has no zones defined."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If

        'we should check status here to see if sensor reports rain or if the controller is busy if so we block
        devStat = getStatusRet(devnamestr)
        'need to start after the name; the name could contain "rain" 6-10-2012
        ij = InStr(LCase(devStat), "status")
        If ij > 0 Then devStat = Mid(devStat, ij)

        If InStr(LCase(devStat), "rain") > 0 Then
            mess = "The controller " & devnamestr & " is reporting Rain. The cycle will not execute."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        If (InStr(LCase(devStat), "running") > 0 Or InStr(LCase(devStat), "waiting") > 0) Then
            mess = "The controller " & devnamestr & " is already Running or Waiting. The cycle will not execute."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If


        'establish a zonelimit so we don't send time to undefined zones

        pary(1) = Trim(pary(1))
        'no need to test controller name and pw - they are retrieved from stored devices
        'now get the named cycle
        cyclestr = getNamedCycle(pary(1))
        If Len(cyclestr) = 0 Then
            mess = "Error: No cycle information found"
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        If Mid(cyclestr, 1, 2) = "X-" Then
            mess = "Error: " & Mid(cyclestr, 3)
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        'ensure the cyclestr is correctly formatted, need to see 8 values
        testary = Split(cyclestr, ":")
        i = UBound(testary)
        If UBound(testary) <> 8 Then
            mess = "Error: " & pary(1) & " cycle is malformed: " & cyclestr
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        'impose the zone limit - zero's all time above the zone limit
        cyclestr = ""
        For i = 0 To 7
            If i < zonelimit Then
                cyclestr = cyclestr & testary(i)
            Else
                cyclestr = cyclestr & "0"
                testary(i) = "0"  'need to zero out this for setback below
            End If
            If i < 7 Then
                cyclestr = cyclestr & ":"
            End If
        Next

        'we can now be sure that cyclestr is properly formed
        'now we need to see if we need to apply the setback
        If testary(8) = "SB" Then
            'get to get the date and the setback
            ipercent = getSetBackMonth() 'gets the setback for the month we are in
            'multiply the values by the percent to get new values
            If ipercent >= 0 Then
                For i = 0 To 7
                    j = (Convert.ToInt32(testary(i)) * ipercent) \ 100
                    If ipercent > 0 And j = 0 And Val(testary(i)) > 0 Then
                        j = 1  'this is the minimum if setback is not zero
                    End If
                    testary(i) = Trim(Str(j))
                Next
                cyclestr = ""
                For i = 0 To 7
                    cyclestr = cyclestr & testary(i)
                    If i < 7 Then cyclestr = cyclestr & ":"
                Next
            Else
                mess = "Error:  Setback not found; Cycle will run at 100%."
                hs.WriteLog("EthrRn:RunNaCy", mess)
            End If
        End If
        icmdstr = "xi=0:" & cyclestr
        'now  get the controller IP address
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            sharedmsg = "Error: " & sharedmsg
            hs.WriteLog("EthrRn:RunNaCy", sharedmsg)
            runNamedCycleRet = sharedmsg
            Exit Function
        End If
        dcount = erdevices.Length
        If dcount < 1 Then
            mess = "Error:  No EtherRain controllers were found."
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If

        'mess = Str(dcount) & "controllers found"
        'hs.WriteLog("EtherRain:runCycle", mess)
        foundboo = False
        For i = 0 To dcount - 1
            If LCase(devnamestr) = LCase(erdevices(i).devname) Then
                foundboo = True
                dsel = i
            End If
        Next
        If Not foundboo Then
            mess = "Error: EtherRain named " & devnamestr & " was not Found. Controllers found: " & Str(dcount - 1)
            hs.WriteLog("EthrRn:RunNaCy", mess)
            runNamedCycleRet = mess
            Exit Function
        End If
        'now send the irrigate command, the following routine will check to 
        'ensure the device is ready, and will login in
        'ensure any error message to put to the log

        'setup the url
        myurl = "http://" & erdevices(dsel).ipPort

        'send the command
        result = doSendCmd(icmdstr, devpwstr)

        'result returned will be X-  or OK
        'replace the X- with:  Error - 
        If InStr(result, "X-") > 0 Then
            result = "CMD NOT Delivered - " & devnamestr & " " & Mid(result, 3)
        ElseIf InStr(result, "OK") > 0 Then
            'construct the time string
            result = "Irrigation CMD Delivered to " & devnamestr & "  Cycle (" & pary(1) & ") Issued: " & cyclestr
        End If
        hs.WriteLog("EthrRn:RunNaCy", result)
        runNamedCycleRet = result
    End Function
    Private Function getNamedCycle(ByVal cycleName As String) As String
        'this private function returns the cycle information of the provided cycleName
        'this function is called by runNamedCycle
        'returns string containing the following:
        '  if the Named cycle was found: returns colon separated zone minutes string
        '  if the Named cycle not found: returns "X-" and an error message
        'this function is derived from  function ReadCycleParameters in the
        'er_cyclebuilder.asp web page

        Dim i, j As Integer
        Dim kstr As String
        Dim erVar As String
        Dim cycles() As String   'this is a one dimensional array
        Dim params() As String  'this is a one dimensional array
        Dim items() As String
        Dim section As String

        erVar = ""
        section = hs.GetINISection("stored cycles", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "ercycles=")
                If j > 0 Then
                    erVar = Mid(items(i), j + 9)
                    Exit For
                End If
            Next
        End If
        'Response.Write "<p><b>ER Var length is : " & len(erVar) & "bytes)</b></p><hr>"
        'Response.Write "<p><b>ER Var is : " & erVar & "</b></p><hr>"

        If Len(erVar) = 0 Then
            'the 
            getNamedCycle = "X-No Cycles Found"
            Exit Function
        End If
        'parsing:  the cycles are separated by "|"   the values are separated by ";"
        kstr = ""
        cycles = Split(erVar, "|")
        For i = 0 To UBound(cycles)
            params = Split(cycles(i), ";")
            If Trim(params(0)) = cycleName Then
                For j = 1 To UBound(params)
                    kstr = kstr & params(j)
                    If j < UBound(params) Then
                        kstr = kstr & ":"
                    End If
                Next
                Exit For
            End If
        Next
        If Len(kstr) = 0 Then
            kstr = "X-Named Cycle Not Found"
        End If
        getNamedCycle = kstr

    End Function
    Public Sub runCycle(ByVal pstr As Object)
        'homeSeer events can only call subroutines directly
        'while the homeseer RunEx command calls functions
        'this sub is a wrapper for the function
        'the sub is called from a homeseer event action
        'the function is called from the homeseer RunEX function
        Dim i, j, istatus As Integer
        Dim rstr As String
        Dim stVar, mess As String
        Dim section As String
        Dim items() As String
        Dim tparams() As String
        Dim suspenddate As String
        Dim d_today, d_sus As Date

        'this test was added to runCycle on 8-5-2011 so run cycles will be controlled globally
        'the quick cycle will still work even if block because it makes a function call
        'disable and suspend have now been integrated into the EtherRain
        'interface.  before sending the request on, get the system status
        'to make sure it is enabled.
        'if diabled or suspended, send a log entry indicating so.

        stVar = ""
        section = hs.GetINISection("system status", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "status=")
                If j > 0 Then
                    stVar = Mid(items(i), j + 7)
                    Exit For
                End If
            Next
        End If
        'split the status str into the global vars and construct statusstr
        istatus = 0      'system is enabled
        suspenddate = ""  'no date
        If Len(stVar) > 0 Then
            tparams = Split(stVar, ";")
            If UBound(tparams) = 1 Then
                istatus = CInt(tparams(0))
                suspenddate = tparams(1)
            End If
            If UBound(tparams) = 0 Then
                istatus = CInt(tparams(0))
            End If
        End If

        'If istatus = 0 Then  we are enabled
        If istatus = 1 Then
            'check the date, if today's date is equal or greater, then continue
            'statusstr = "Suspended by User until " & suspenddate
            d_today = Date.Now
            d_sus = CDate(suspenddate)
            If d_today < d_sus Then
                mess = "EtherRain Irrigation is Currently Suspended by User"
                hs.WriteLog("EthrRn:RunCy", mess)
                Exit Sub
            End If
        ElseIf istatus = 2 Then
            mess = "EtherRain Irrigation is Currently Disabled by User"
            hs.WriteLog("EthrRn:RunCy", mess)
            Exit Sub

        End If

        'Dim rstr As String

        rstr = runCycleRet(pstr)

    End Sub

    Public Function runCycleRet(ByVal pStr As Object) As String

        ' runCycle sends a command to an EtherRain Valve Controller
        ' The parameter string contains 10 parameters each separated by a semicolon
        ' The parameters are:  Controller name, controller password, and 8 valve "on" times.
        ' (Controller name; controller password; m1; m2; m3; m4; m5; m6; m7; m8 )
        '
        ' All ten parameters are required.  If ten parameters are not found in the 
        'parameter string, the event terminates with an error written to the log.
        '
        ' The parameters m1 through m8 must be whole numbers from 0 to 99.  These 
        ' numbers represent valve "on" times in minutes.  For example, if the m1 
        ' parameter was 25, then valve 1 would open for 25 minutes.
        '
        ' To skip a valve, enter 0 for it's valve on time.
        ' 6-02-2012  - if sensor reporting rain or if already running/waiting, then log message and return; no execution
        ' 11-05-2012  - fixed bug: status check was overwriting the ip address; a new ipaddress variable was added
        ' 11-05-2012  - moved the check status before the getevirodevices command to fix above 
        '			  - this was not an issue in runNamedCycelRet because the status check was done before the getevirodevices
        ' parse the pStr here

        Dim i, dsel As Integer
        Dim wstr, mess As String
        Dim devnamestr, devpwstr, icmdstr, result As String
        Dim devStat As String
        Dim dcount As Integer
        Dim errboo, foundboo As Boolean

        'cannot use commas as the delimiter because VB parses into multiple params
        'use semi-colons instead
        dsel = 0  'dsel is device selected

        ' Split the string at the comma characters and add each field to a ListBox
        Dim pary() = pStr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length


        If (i <> 10) Then
            mess = "Parameter Mismatch, " & Str(i) & " provided, 10 required."
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If
        'now, count the array, are there 10 members?
        'if yeah, then get controller name and password
        'do these meet the requirements for controller name and password?
        'if yes, then get the minutes array and construct the irrigation string
        devnamestr = Trim(pary(0))
        devpwstr = Trim(pary(1))
        If devnamestr.Length > 16 Then
            mess = "Error: ER Device Name too Long in parameter string"
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If
        If devnamestr.Length < 4 Then
            mess = "Error: ER Device Name too Short in parameter string"
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If
        If devpwstr.Length > 5 Then
            mess = "Error: ER Device Password Long in parameter string"
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If

        icmdstr = "xi=0"
        errboo = False
        For i = 2 To 9
            wstr = Trim(pary(i))
            If IsNumeric(wstr) Then
                icmdstr = icmdstr & ":" & Trim(pary(i))
            Else
                errboo = True
            End If
        Next
        If (errboo) Then
            mess = "Error: Some supplied Valve Times are non-numeric."
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If

        'we should check status here to see if sensor reports rain or if the controller is busy if so we block
        devStat = getStatusRet(devnamestr)

        'need to start after the name; the name could contain "rain" 6-10-2012
        i = InStr(LCase(devStat), "status")
        If i > 0 Then devStat = Mid(devStat, i)

        If InStr(LCase(devStat), "rain") > 0 Then
            mess = "The controller " & devnamestr & " is reporting Rain. The cycle will not execute."
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If
        If (InStr(LCase(devStat), "running") > 0 Or InStr(LCase(devStat), "waiting") > 0) Then
            mess = "The controller " & devnamestr & " is already Running or Waiting. The cycle will not execute."
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If

        'now  get the controller IP address
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            sharedmsg = "Error: " & sharedmsg
            hs.WriteLog("EthrRn:RunCy", sharedmsg)
            runCycleRet = sharedmsg
            Exit Function
        End If
        dcount = erdevices.Length
        If dcount < 1 Then
            mess = "Error:  No EtherRain controllers were found."
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If

        'mess = Str(dcount) & "controllers found"
        'hs.WriteLog("EtherRain:runCycle", mess)
        foundboo = False
        For i = 0 To dcount - 1
            mess = erdevices(i).devname & " " & erdevices(i).ipPort
            'hs.WriteLog("EtherRain:runCycle", mess)
            If LCase(devnamestr) = LCase(erdevices(i).devname) Then
                'mess = "CONTROLLER FOUND"
                foundboo = True
                'hs.WriteLog("EtherRain:runCycle", mess)
                dsel = i
            End If
        Next
        If Not foundboo Then
            mess = "Error: EtherRain named " & devnamestr & " was not Found. Controllers found: " & Str(dcount - 1)
            hs.WriteLog("EthrRn:RunCy", mess)
            runCycleRet = mess
            Exit Function
        End If



        'now send the irrigate command, the following routine will check to ensure the device is ready, and will login
        'ensure any error message to put to the log

        'setup the url
        myurl = "http://" & erdevices(dsel).ipPort

        'send the command
        result = doSendCmd(icmdstr, devpwstr)

        'result returned will be X-  or OK
        'replace the X- with:  Error - 
        If InStr(result, "X-") > 0 Then
            result = "CMD NOT Delivered - " & devnamestr & " " & Mid(result, 3)
        ElseIf InStr(result, "OK") > 0 Then
            'construct the time string
            wstr = ""
            For i = 2 To 9
                wstr = wstr & Trim(pary(i))
                If i < 9 Then wstr = wstr & ":"
            Next
            result = "Irrigation CMD Delivered to " & devnamestr & "  Cycle Issued: " & wstr
        End If
        hs.WriteLog("EthrRn:RunCy", result)
        runCycleRet = result
    End Function


    Public Sub stopAllCycles(ByVal str As String)
        'this is just a wrapper so an event can call the function
        Dim rstr As String
        rstr = ""
        stopAllCyclesRet(rstr)


    End Sub
    Public Function stopAllCyclesRet(ByVal pstr As String) As String
        'this sends a stop command to all controllers found.
        'no parameters are required
        'this function is called from er_home "Disable"
        'can be called from a HomeSeer Manual event
        'or a weather event

        Dim i, j, k, dsel As Integer
        Dim wstr, qstr, mess As String
        Dim devpwstr, url As String
        Dim dcount As Integer
        Dim params() As String
        Dim rez As Boolean

        webstr = ""
        stopAllCyclesRet = "ok"

        'get the devices from the ini file, you need the passwords
        dcount = readControllers()  'this reads the hs. controller definitions
        If dcount = 0 Then
            mess = "No EtherRain Controllers defined in HomeSeer."
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Function
        End If
        'controlAry(i) contains HS defined controllers
        ' when split, params(0) is name; params(1) is pw

        'send a reset xr command to all devices
        'now  get the controller IP addressess
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            sharedmsg = "Error: " & sharedmsg
            hs.WriteLog("EthrRn:StopCy", sharedmsg)
            Exit Function
        End If
        dcount = erdevices.Length
        If dcount < 1 Then
            mess = "Error:  No EtherRain controllers were found."
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Function
        End If

        For i = 1 To dcount - 1
            'now send the stop command to all found controllers, but only
            'if the password is available from the HS ini file
            'get the password
            devpwstr = ""
            For j = 0 To UBound(controlAry)
                params = Split(controlAry(j), ";")
                If Trim(erdevices(i).devname) = Trim(params(0)) Then
                    If UBound(params) > 0 Then
                        devpwstr = Trim(params(1))
                        Exit For
                    End If
                End If

            Next
            If Len(Trim(erdevices(i).devname)) = 0 Then
                Continue For
            End If
            'setup the url
            myurl = "http://" & Trim(erdevices(i).ipPort)
            'also, make sure the password is accepted
            qstr = "/" & configPage & "?lu=admin&lp=" & devpwstr
            'we must first log in before sending the change command
            url = myurl & qstr
            rez = getPage(url)
            'check to see if the page was set by filtering through the page
            ' txtresponse.Text = webstr
            If Not rez Then
                mess = "No Response from " & erdevices(i).devname & " (" & erdevices(i).ipPort & ")"
                hs.WriteLog("EthrRn:StopCy", mess)
                Continue For
            End If
            'we got a response, take a look at it
            'check for ur: Admin, if not:
            k = InStr(webstr, "admin")
            If k = 0 Then
                mess = "Password not Accepted by" & erdevices(i).devname & " Check Password"
                hs.WriteLog("EthrRn:StopCy", mess)
                Continue For
            End If

            ' qstr = "/" & resultPage & "?&xr=1"
            ' reset command should be just xr and no "&" if first command in string
            qstr = "/" & resultPage & "?xr"
            url = myurl & qstr
            rez = getPage(url)
            'check to see if the page was set by filtering through the page
            ' txtresponse.Text = webstr
            If Not rez Then
                mess = "No Response from " & erdevices(i).devname & " (" & erdevices(dsel).ipPort & ")"
                hs.WriteLog("EthrRn:StopCy", mess)
                Continue For
            End If
            'confirm that command was accepted
            k = InStr(webstr, "cs:")
            'check operating status
            wstr = "?"
            If k > 0 Then
                wstr = Mid(webstr, k + 4, 2)
            End If
            If wstr <> "OK" Then
                mess = "Stop command was NOT accepted by " & erdevices(i).devname & " (" & erdevices(i).ipPort & ")"
                hs.WriteLog("EthrRn:StopCy", mess)
                Continue For
            End If

            mess = "Stop command issued to " & erdevices(i).devname & " (" & erdevices(i).ipPort & ")"
            hs.WriteLog("EthrRn:StopCy", mess)

        Next
        mess = "stopAllCycles command completed."
        hs.WriteLog("EthrRn:StopCy", mess)

    End Function
    Public Sub stopCycle(ByVal pstr As Object)

        'this turns the controller off
        'pstr must contaian the controller name and password
        Dim i, k, dsel As Integer
        Dim wstr, qstr, mess As String
        Dim devnamestr, devpwstr, url As String
        Dim dcount As Integer
        Dim foundboo, rez As Boolean

        webstr = ""

        'send a reset xr=1 command to the selected device
        ' 7-22-2017 the reset command is just xr
        'cannot use commas as the delimiter because VB parses into multiple params
        'use semi-colons instead

        ' Split the string at the comma characters and add each field to a ListBox
        Dim pary() = pstr.ToString.Split(Convert.ToChar(";"))
        i = pary.Length
        If (i <> 2) Then
            mess = "Parameter Error.  A controller name and password are required (separated by ;)."
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        devnamestr = Trim(pary(0))
        devpwstr = Trim(pary(1))
        If devnamestr.Length > 16 Then
            mess = "Error: ER Device Name too Long in parameter string"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        If devnamestr.Length < 4 Then
            mess = "Error: ER Device Name too Short in parameter string"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        If devpwstr.Length > 5 Then
            mess = "Error: ER Device Password Long in parameter string"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If

        'now  get the controller IP address
        sharedmsg = ""
        If (getEviroDevices() = False) Then
            sharedmsg = "Error: " & sharedmsg
            hs.WriteLog("EthrRn:StopCy", sharedmsg)
            Exit Sub
        End If
        dcount = erdevices.Length
        If dcount < 1 Then
            mess = "Error:  No EtherRain controllers were found."
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If

        foundboo = False
        For i = 0 To dcount - 1
            mess = erdevices(i).devname & " " & erdevices(i).ipPort
            If LCase(devnamestr) = LCase(erdevices(i).devname) Then
                'mess = "CONTROLLER FOUND"
                foundboo = True
                'hs.WriteLog("EtherRain:runCycle", mess)
                dsel = i
            End If
        Next
        If Not foundboo Then
            mess = "Error: EtherRain named " & devnamestr & " was not Found. Controllers found: " & Str(dcount - 1)
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        'now send the stop command, the following routine will check to 
        'ensure the device is ready, and will login in
        'ensure any error message to put to the log

        'setup the url
        myurl = "http://" & erdevices(dsel).ipPort
        'also, make sure the password is accepted
        qstr = "/" & configPage & "?lu=admin&lp=" & devpwstr
        'we must first log in before sending the change command
        url = myurl & qstr
        rez = getPage(url)
        'check to see if the page was set by filtering through the page
        ' txtresponse.Text = webstr
        If Not rez Then
            mess = "Error:  No Response from " & devnamestr & " (" & erdevices(dsel).ipPort & ")"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        'we got a response, take a look at it
        'check for ur: Admin, if not:
        k = InStr(webstr, "admin")
        If k = 0 Then
            mess = "Error:  Password not Accepted. Check Password"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If

        'qstr = "/" & resultPage & "?&xr=1"
        qstr = "/" & resultPage & "?xr"
        url = myurl & qstr
        rez = getPage(url)
        'check to see if the page was set by filtering through the page
        ' txtresponse.Text = webstr
        If Not rez Then
            mess = "Error:  No Response from " & devnamestr & " (" & erdevices(dsel).ipPort & ")"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If
        'confirm that command was accepted
        k = InStr(webstr, "cs:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr <> "OK" Then
            mess = "Error:  Stop command was not accepted by " & devnamestr & " (" & erdevices(dsel).ipPort & ")"
            hs.WriteLog("EthrRn:StopCy", mess)
            Exit Sub
        End If

        mess = "Stop command issued to " & devnamestr & " (" & erdevices(dsel).ipPort & ")"
        hs.WriteLog("EthrRn:StopCy", mess)


    End Sub



    Public Function convertToInt(ByVal toconvert As String) As Integer

        Try
            convertToInt = CInt(toconvert)
        Catch ex As Exception
            convertToInt = 0
        End Try



    End Function
    Public Function doSendCmd(ByVal cmd As String, ByVal password As String) As String
        Dim k As Integer
        Dim qstr, wstr As String
        Dim url As String
        Dim rez As Boolean

        'this function sends the irrigate command for the Scheduler service
        'returns the command results
        'PRErequisite:   myurl has been set
        webstr = ""

        'also, make sure the password is accepted
        qstr = "/" & configPage & "?lu=admin&lp=" & password
        'we must first log in before sending the change command
        url = myurl & qstr
        rez = getPage(url)
        'check to see if the page was set by filtering through the page
        ' txtresponse.Text = webstr
        If Not rez Then
            doSendCmd = "X-No Response"
            Exit Function
        End If
        'we got a response, take a look at it
        'check for ur: Admin, if not:
        k = InStr(webstr, "admin")
        If k = 0 Then
            doSendCmd = "X-Password Failed"
            Exit Function
        End If

        'look at the results to see if os: RD
        qstr = "/" & resultPage & "?" & "xs"
        url = myurl & qstr
        rez = getPage(url)
        If Not rez Then
            doSendCmd = "X-No Status Returned"
            Exit Function
        End If
        k = InStr(webstr, "os:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr = "BZ" Or wstr = "WT" Then
            doSendCmd = "X-Busy or Waiting"
            Exit Function
        End If
        'everything ready, so issue xi command, 
        qstr = "/" & resultPage & "?" & cmd
        url = myurl & qstr
        rez = getPage(url)
        If Not rez Then
            doSendCmd = "X-Irrigation command reception was not confirmed."
            Exit Function
        End If
        k = InStr(webstr, "cs:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr <> "OK" Then
            doSendCmd = "X-CMD Not Accepted"
            Exit Function
        End If
        k = InStr(webstr, "rz:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
        End If
        If wstr = "SH" Then
            doSendCmd = "X-Reporting Short"
            Exit Function
        End If
        k = InStr(webstr, "rn:")
        'check operating status
        wstr = "?"
        If k > 0 Then
            wstr = Mid(webstr, k + 4, 2)
            If wstr = "1" Then
                doSendCmd = "X-Halted by Rain"
                Exit Function
            End If
        End If

        doSendCmd = "OK-CMD Delivered"

    End Function

    Public Function getEviroDevices() As Boolean

        Dim discoPort As Integer
        Dim sendbuf() As Byte
        Dim receivebuf As String

        'Dim enc As New System.Text.ASCIIEncoding
        Dim strData, devstr As String
        Dim bufRet As Integer
        Dim frpntr, packetstart, packetend As Integer
        'Dim packetstr As String
        Dim lflag As Boolean
        Dim i, j, k, ij, ik As Integer
        Dim ipaddr, pname, ptype, prt As String
        Dim erflag As Boolean
        Dim kstr, nstr As String

        'Dim devices() As deviceStruct

        'gets all the active devices and puts them in devices()
        kstr = ""
        ipaddr = ""
        receivebuf = ""
        discoPort = defRport                'const defined in global
        sendbuf = strToByteArray(discocmd)  'const defined in global

        'put this in a loop.  Sockets are not being granted sometimes 7-25-09
        bufRet = 0
        For i = 1 To 15
            bufRet = UDPbroadcastEviro(sendbuf, receivebuf, discoPort)
            If bufRet <> -2 Then
                Exit For
            End If
        Next
        If bufRet = -1 Then
            sharedmsg = "Problem Sending Data.  Check Your Network Connection"
            Return False
        End If
        If bufRet = -2 Then
            sharedmsg = "Could Not Broadcast (Exception Error)"
            Return False
        End If

        'checking first that the buffer has something in it
        If bufRet < -2 Then
            sharedmsg = "Unknown Error while broadcasting"
            Return False
        End If
        'receivebuf will either have nothing, one or many packets
        'packets have format==> :FR:nnn.nnn.nnn.nnn:response
        'no space separates multiple packets, so search for the header ":FR:"
        deviceTot = 0

        frpntr = 0
        lflag = True
        While lflag
            frpntr = InStr(frpntr + 1, receivebuf, ":FR:")
            If frpntr = 0 Then
                Exit While
            End If
            'now decode here
            packetstart = frpntr + 4
            packetend = InStr(packetstart, receivebuf, ":FR:")
            If packetend = 0 Then
                packetend = Len(receivebuf)
                lflag = False
            End If

            strData = Mid(receivebuf, packetstart, packetend - packetstart)
            'ip address ends after next ":"
            k = InStr(strData, ":")
            If k > 0 Then
                ipaddr = Mid(strData, 1, k - 1)
                erflag = False
            Else
                erflag = True
            End If
            'process the results here
            ptype = "???"
            pname = "???"
            prt = "???"
            k = InStr(strData, productkey)
            If k > 0 Then  'have a return
                'extract the p=  ; n= and a=  values
                i = InStr(strData, "t=")
                j = InStr(strData, "n=")
                ij = InStr(strData, "p=")
                ik = InStr(strData, "a=")
                If i > 0 And j > i + 3 Then
                    ptype = "Type: " & Mid(strData, i + 2, j - i - 3)
                End If
                If j > 0 And (ij - 2) > j Then
                    pname = "Name: " & Mid(strData, j + 2, ij - j - 3)
                    kstr = Mid(strData, j + 2, ij - j - 3)
                End If
                If ij > 0 And (ik - 2) > ij Then
                    'prt = "Port: " & Mid(strData, ij + 2, ik - ij - 3)
                    prt = Mid(strData, ij + 2, ik - ij - 3)
                End If
                If Not erflag Then
                    'only list the etherrain devices
                    If InStr(ptype, "ER") > 0 Then
                        'devstr = pname & "  " & " IP: " & ipaddr & ":" & prt
                        devstr = pname & "  " & " (" & ipaddr & ":" & prt & ")"
                        nstr = ipaddr & ":" & prt
                        'If lstDevices.FindString(devstr) = -1 Then
                        'lstDevices.Items.Add(devstr)
                        'End If
                        deviceTot = deviceTot + 1
                        ReDim Preserve erdevices(deviceTot)
                        erdevices(deviceTot) = New deviceStruct()
                        erdevices(deviceTot).devname = kstr
                        erdevices(deviceTot).ipPort = nstr
                        erdevices(deviceTot).status = True
                        erdevices(deviceTot).del = False
                    End If
                End If
            End If
        End While

        'If lstDevices.Items.Count > 0 Then
        'cmdaction(0).Enabled = True
        'lstDevices.SelectedIndex = 0
        'Exit Function
        'End If

        getEviroDevices = True


    End Function
    Public Function UDPbroadcastEviro( _
                  ByVal sBuff() As Byte, ByRef rDstr As String, _
                  ByVal Port As Integer) As Integer     'Returns # bytes received 
        Dim retstat As Integer
        'below line was a tenscripting change - they don't import Net
        Dim Sck As Sockets.Socket
        Dim Due As DateTime
        Dim Encrp As IPEndPoint
        'Dim wstr As String
        Dim rbuff() As Byte
        Dim mess As String
        'Dim bcAdrStr As String
        'bcAdrStr = "192.168.1.255"
        'bcAdrStr = "192.168.255.255"
        'Dim bcAdr As IPAddress = IPAddress.Parse(bcAdrStr)
        Dim sender As New IPEndPoint(IPAddress.Any, 0)
        Dim tempRemoteEP As EndPoint = CType(sender, EndPoint)
        Dim enc As New System.Text.ASCIIEncoding
        'Dim sizetestS, sizetestR As Integer
        Sck = New Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp)

        Try
            'Sck.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, 10000)
            Sck.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, 10000)
            Sck.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, 10000)
            Sck.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.Broadcast, 1)
            Encrp = New IPEndPoint(IPAddress.Broadcast, Port)
            'Encrp = New IPEndPoint(bcAdr, Port)
            'sizetestS = sBuff.Length
            retstat = Sck.SendTo(sBuff, 0, sBuff.Length, SocketFlags.None, Encrp)
            If retstat > 0 Then
                Due = Now.AddMilliseconds(1000) '7-25-09 change to 1 second from .8 second time-out, it's just the local net 
                retstat = -3 'timed-out 
                Do While Now < Due
                    If Sck.Available > 0 Then
                        'getting "Specified argument was out of the range of valid values" exception errors
                        'occassionally - so try the sck.availabe without the -1
                        'ReDim rbuff(Sck.Available - 1)
                        'ReDim rbuff(Sck.Available)
                        'this 2k buffer seems to have solved the random buffer exception errors
                        ReDim rbuff(2048)
                        'sizetestR = Sck.Available
                        retstat = Sck.ReceiveFrom(rbuff, 0, Sck.Available, SocketFlags.None, tempRemoteEP)
                        'wstr = CType(tempRemoteEP, IPEndPoint).Address.ToString()
                        'str = enc.GetString(rbuff)
                        rDstr = rDstr & ":FR:" & CType(tempRemoteEP, IPEndPoint).Address.ToString() & ":" & _
                                enc.GetString(rbuff)
                    End If
                Loop

            Else
                retstat = -1 ' fail on send 
            End If
        Catch ex As Exception
            'General Exception received--add code here to analyze the exception. A messagebox would be one idea. 
            mess = ex.Message  '& "sbuflen: " & sizetestS & " rbuflen: " & sizetestR
            hs.WriteLog("EthrRn:UDP", mess)
            retstat = -2
        Finally
            Sck.Close() 'Always close the socket when done. 
        End Try

        Return retstat
    End Function

    Public Function strToByteArray(ByVal str As String) As Byte()
        Dim encoding As New System.Text.ASCIIEncoding
        Return encoding.GetBytes(str)

    End Function

    Public Function getPage(ByVal url As String) As Boolean

        Dim request As WebRequest
        Try
            request = WebRequest.Create(url)
        Catch
            getPage = False
            Exit Function
        End Try

        Dim response As WebResponse
        'put an exception handler around this statement:
        Try
            response = request.GetResponse()
        Catch ex As Exception
            getPage = False
            Exit Function
        End Try
        Dim reader As StreamReader = New StreamReader(response.GetResponseStream())
        Dim str As String = reader.ReadLine()
        Dim flag As Boolean = False
        webstr = ""
        Do While Not flag
            webstr = webstr & str
            str = reader.ReadLine()
            If (str Is Nothing) Then
                flag = True
            End If
        Loop
        If Len(webstr) > 0 Then
            getPage = True
        Else
            getPage = False
        End If


    End Function
    Public Function getPageReset(ByVal url As String) As Boolean

        'this just sends the request but doesn't try to read
        ' used in config when resetting the device, no response to come back!

        Dim request As WebRequest

        Try
            request = WebRequest.Create(url)
        Catch
            getPageReset = False
            Exit Function
        End Try
        request.Timeout = 1000 'one second
        Dim response As WebResponse
        'put an exception handler around this statement:
        Try
            response = request.GetResponse()
        Catch ex As Exception
            getPageReset = False
            Exit Function
        End Try




        getPageReset = True




    End Function

    Private Function readControllers() As Integer
        'gets all of the controller information
        'returns number of controllers currently defined
        Dim i, j As Integer
        Dim items() As String
        Dim controllers() As String
        Dim section As String
        Dim cfound As Integer

        Dim cVar As String

        cVar = ""
        section = hs.GetINISection("stored controllers", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "controllers=")
                If j > 0 Then
                    cVar = Mid(items(i), j + 12)
                    Exit For
                End If
            Next
        End If
        If Len(cVar) > 0 Then
            'Response.Write "<p><b>cVar is : " & cVar & "</b></p><hr>"
            controllers = Split(cVar, "|")
            cfound = UBound(controllers) + 1
            'Response.Write "<p><b>number found is : " & cfound & "</b></p><hr>"
            ReDim controlAry(cfound - 1)
            For i = 0 To cfound - 1
                controlAry(i) = controllers(i)
            Next
        Else
            cfound = 0
        End If


        'Response.Write "<p><b>ER Var length is : " & len(erVar) & "bytes)</b></p><hr>"
        'Response.Write "<p><b>ER Var is : " & erVar & "</b></p><hr>"
        readControllers = cfound

    End Function

    Function readAlertConfig() As String
        'this reads the alert string and returns the string

        'this function duplicated on home page
        Dim aVar As String
        Dim i, j As Integer
        Dim section As String
        Dim items() As String

        'get the following
        'start time, in minutes; convert to string hours:minutes and AM/PM
        'get the email address
        'get the email option (0 or 1)
        'get the log option (0 or 1)

        aVar = ""
        section = hs.GetINISection("alerts_config", "EtherRain.ini")

        'Response.Write "<p><b>section " & section & "</b></p><hr>"

        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                'Response.Write "<p><b>Item " & items(i) & "</b></p><hr>"
                j = InStr(items(i), "eralerts=")
                If j > 0 Then
                    aVar = Mid(items(i), j + 9)
                    Exit For
                End If
            Next
        End If

        readAlertConfig = aVar

    End Function

    Private Function getSetBackMonth() As Integer

        'if negative, no value found
        'this vbscript function is executed on the server
        Dim i, j, k As Integer
        Dim erVar As String
        Dim items(12) As String
        Dim section As String
        Dim sbfound As Boolean
        Dim sb(12) As String
        Dim sbi As Integer


        sbfound = False
        erVar = ""
        section = hs.GetINISection("stored setbacks", "EtherRain.ini")
        If Len(section) > 0 Then
            items = Split(section, Chr(0))
            For i = 0 To UBound(items)
                j = InStr(items(i), "ersetback=")
                If j > 0 Then
                    erVar = Mid(items(i), j + 10)
                    Exit For
                End If
            Next
        End If
        If Len(erVar) > 0 Then 'parse the storage variable here
            'the values are separated by ";"
            items = Split(erVar, ";")
            If UBound(items) = 11 Then
                sbfound = True
                For i = 0 To 11
                    sb(i) = items(i)
                Next
            End If
        End If
        If sbfound = False Then
            getSetBackMonth = -1
            Exit Function
        End If
        'now get the month
        k = Now.Month
        sbi = -1
        If UBound(items) >= k - 1 Then
            sbi = Convert.ToInt32(items(k - 1))
        End If
        getSetBackMonth = sbi

    End Function

    Public Sub disableSystem(ByVal str As String)
        'this lets you disable the system from an event
        'this is a wrapper for the setSystemStatus function
        Dim pstr As String
        Dim rez As Boolean
        pstr = "disable"
        rez = setSystemStatus(pstr)


    End Sub

    Public Sub enableSystem(ByVal str As String)
        'this lets you re-enable the sytem irrigation from an event
        Dim pstr As String
        Dim rez As Boolean
        pstr = "resume"
        rez = setSystemStatus(pstr)


    End Sub

    Public Sub suspendSystem(ByVal dstr As String)
        'this lets you suspend the system irrigation from an event
        Dim i As Integer
        Dim wstr As String
        'check that str is value less than 10
        Dim pstr As String
        Dim rez As Boolean

        i = Convert.ToInt32(Trim(dstr))
        If i > 9 Then i = 9
        If i = 0 Then i = 1
        wstr = Trim(Str(i))
        pstr = "suspend;" & wstr
        rez = setSystemStatus(pstr)



    End Sub

    Private Function setSystemStatus(ByVal str As String) As Boolean
        Dim statstr As String
        Dim tparams() As String
        Dim days As Integer
        Dim d_today, d_added As Date
        Dim mess As String

        setSystemStatus = True  'calls to this function dont test the return

        ' runNamedCycle and runCycle checks system status through ini section "system status" "status"
        ' stores statstr with format:  istatus(0-2) ; suspenddate (str with format mm-dd-yyyy)
        ' str will be:  "disable"  "suspend;n" (where n is days) "resume"  "enable"

        If str = "enable" Or str = "resume" Then
            hs.ClearINISection("system status", "EtherRain.ini")
            mess = "System Enabled or Resumed by User Event."
            hs.WriteLog("EthrRn:Alerts", mess)
        End If
        If str = "disable" Then
            d_today = Date.Now
            statstr = "2;" & d_today
            hs.SaveINISetting("system status", "status", statstr, "EtherRain.ini")
            'now all issue immediate stop
            stopAllCyclesRet("")
            ' now log this
            mess = "System Disabled by User Event."
            hs.WriteLog("EthrRn:Alerts", mess)
        End If
        If InStr(str, "suspend") > 0 Then
            tparams = Split(str, ";")
            If UBound(tparams) = 1 Then
                days = CInt(tparams(1))
            Else
                days = 1
            End If
            'need to do some date arithmetic here
            d_today = Date.Now
            d_added = DateAdd(DateInterval.Day, days, d_today)
            statstr = "1;" & d_added
            hs.SaveINISetting("system status", "status", statstr, "EtherRain.ini")
            mess = "System Suspended until " & d_added & " by User Event."
            hs.WriteLog("EthrRn:Alerts", mess)
        End If
        ' Response.Write "Today date is " & d_today & "<br>"

    End Function


